@@ -21,6 +21,7 @@ import moviepy.audio.io as aio
 from .io.ffmpeg_writer import ffmpeg_write_image, ffmpeg_write_video
 from .io.ffmpeg_reader import ffmpeg_read_image
 from .io.ffmpeg_tools import ffmpeg_merge_video_audio
+from .io.gif_writers import write_gif, write_gif_with_tempfiles
 
 from .tools.drawing import blit
 
@@ -74,7 +75,7 @@ class VideoClip(Clip):
     ismask
       Boolean set to `True` if the clip is a mask.
 
-    get_frame
+    make_frame
       A function ``t-> frame at time t`` where ``frame`` is a
       w*h*3 RGB array.
 
@@ -95,16 +96,19 @@ class VideoClip(Clip):
 
     """
 
-    def __init__(self, ismask=False, get_frame=None):
+    def __init__(self, make_frame=None, ismask=False, duration=None):
         Clip.__init__(self)
         self.mask = None
         self.audio = None
         self.pos = lambda t: (0, 0)
         self.relative_pos = False
-        if get_frame is not None:
-            self.get_frame = get_frame
-            self.size =get_frame(0).shape[:2][::-1]
+        if make_frame is not None:
+            self.make_frame = make_frame
+            self.size =self.get_frame(0).shape[:2][::-1]
         self.ismask = ismask
+        if duration is not None:
+            self.duration = duration
+            self.end = duration
 
     @property
     def w(self):
@@ -384,7 +388,7 @@ class VideoClip(Clip):
     @requires_duration
     def write_gif(self, filename, fps=None, program= 'ImageMagick',
                opt="OptimizeTransparency", fuzz=1, verbose=True,
-               loop=0, dispose=False):
+               loop=0, dispose=False, colors=None, tempfiles=False):
         """ Write the VideoClip to a GIF file.
 
         Converts a VideoClip into an animated GIF using ImageMagick
@@ -427,203 +431,15 @@ class VideoClip(Clip):
             >>> myClip.speedx(0.5).to_gif('myClip.gif')
 
         """
-
-        if fps is None:
-            fps = self.fps
-
-        fileName, fileExtension = os.path.splitext(filename)
-        tt = np.arange(0,self.duration, 1.0/fps)
-
-        tempfiles = []
-
-        verbose_print(verbose, "\nMoviePy: building GIF file %s\n"%filename
-                      +40*"-"+"\n")
-
-        verbose_print(verbose, "Generating GIF frames.\n")
-
-        total = int(self.duration*fps)+1
-        for i, t in tqdm(enumerate(tt), total=total):
-
-            name = "%s_GIFTEMP%04d.png"%(fileName, i+1)
-            tempfiles.append(name)
-            self.save_frame(name, t, savemask=True)
-
-        verbose_print(verbose, "Done generating GIF frames.\n")
-
-        delay = int(100.0/fps)
-
-        if program == "ImageMagick":
-
-            cmd = [IMAGEMAGICK_BINARY,
-                  '-delay' , '%d'%delay,
-                  "-dispose" ,"%d"%(2 if dispose else 1),
-                  "-loop" , "%d"%loop,
-                  "%s_GIFTEMP*.png"%fileName,
-                  "-coalesce",
-                  "-fuzz", "%02d"%fuzz + "%",
-                  "-layers", "%s"%opt,
-                  filename]
-
-        elif program == "ffmpeg":
-
-            cmd = [FFMPEG_BINARY, '-y',
-                   '-f', 'image2', '-r',str(fps),
-                   '-i', fileName+'_GIFTEMP%04d.png',
-                   '-r',str(fps),
-                   filename]
-
-        try:
-            subprocess_call( cmd, verbose = verbose )
         
-        except IOError as err:
-
-            error = ("MoviePy Error: creation of %s failed because "
-              "of the following error:\n\n%s.\n\n."%(filename, str(err)))
-            
-            if program == "ImageMagick":
-                error = error + ("This can be due to the fact that "
-                    "ImageMagick is not installed on your computer, or "
-                    "(for Windows users) that you didn't specify the "
-                    "path to the ImageMagick binary in file conf.py." )
-            
-            raise IOError(error)
-
-        for f in tempfiles:
-            os.remove(f)
-
-
-
-    @requires_duration
-    def write_gif2(self, filename, fps=None, program= 'ImageMagick',
-                   opt="OptimizeTransparency", fuzz=1, verbose=True,
-                   loop=0, dispose=False):
-        """ Write the VideoClip to a GIF file, without temporary files.
-
-        Converts a VideoClip into an animated GIF using ImageMagick
-        or ffmpeg.
-
-
-        Parameters
-        -----------
-
-        filename
-          Name of the resulting gif file.
-
-        fps
-          Number of frames per second (see note below). If it
-            isn't provided, then the function will look for the clip's
-            ``fps`` attribute (VideoFileClip, for instance, have one).
-
-        program
-          Software to use for the conversion, either 'ImageMagick' or
-          'ffmpeg'.
-
-        opt
-          (ImageMagick only) optimalization to apply, either
-          'optimizeplus' or 'OptimizeTransparency'.
-
-        fuzz
-          (ImageMagick only) Compresses the GIF by considering that
-          the colors that are less than fuzz% different are in fact
-          the same.
-
-
-        Notes
-        -----
-
-        The gif will be playing the clip in real time (you can
-        only change the frame rate). If you want the gif to be played
-        slower than the clip you will use ::
-
-            >>> # slow down clip 50% and make it a gif
-            >>> myClip.speedx(0.5).write_gif('myClip.gif')
-
-        """
-        
-        #
-        # We use processes chained with pipes.
-        #
-        # if program == 'ffmpeg'
-        # frames --ffmpeg--> gif
-        #
-        # if program == 'ImageMagick' and optimize == (None, False)
-        # frames --ffmpeg--> bmp frames --ImageMagick--> gif
-        #
-        # 
-        # if program == 'ImageMagick' and optimize != (None, False)
-        # frames -ffmpeg-> bmp frames -ImagMag-> gif -ImagMag-> better gif
-        #
-    
-        if fps is None:
-            fps=self.fps
-        delay= 100.0/fps
-
-        cmd1 = [FFMPEG_BINARY, '-y', '-loglevel', 'error',
-                '-f', 'rawvideo',
-                '-vcodec','rawvideo', '-r', "%.02f"%fps,
-                '-s', "%dx%d"%(self.w, self.h), '-pix_fmt', 'rgb24',
-                '-i', '-']
-        
-        if program == "ffmpeg":
-            proc1 = sp.Popen(cmd1+['-r', "%.02f"%fps, filename],
-                             stdin=sp.PIPE, stdout=DEVNULL)
+        if tempfiles:
+            write_gif_with_tempfiles(self, filename, fps=fps,
+               program= program, opt=opt, fuzz=fuzz, verbose=verbose,
+               loop=loop, dispose= dispose, colors=colors)
         else:
-            proc1 = sp.Popen(cmd1+['-f', 'image2pipe',
-                                   '-vcodec', 'bmp', '-'],
-                             stdin=sp.PIPE, stdout=sp.PIPE)
-
-        if program == 'ImageMagick':
-            
-            cmd2 = [IMAGEMAGICK_BINARY, '-delay', "%.02f"%(delay),
-                    "-dispose" ,"%d"%(2 if dispose else 1),
-                    '-loop', '%d'%loop, '-', '-coalesce']
-            
-            if (opt in [False, None]):
-                proc2 = sp.Popen(cmd2+[filename], stdin=proc1.stdout)
-                
-            else:
-                proc2 = sp.Popen(cmd2+['gif:-'] , stdin=proc1.stdout,
-                                 stdout=sp.PIPE)
-                
-            if opt:
-                
-                cmd3 = [IMAGEMAGICK_BINARY, '-', '-layers', opt,
-                        '-fuzz', '%d'%fuzz+'%',filename]
-                proc3 = sp.Popen(cmd3, stdin=proc2.stdout)
-        
-        # We send all the frames to the first process
-        verbose_print(verbose, "\nMoviePy: building GIF file %s\n"%filename
-                                +40*"-"+"\n")
-        verbose_print(verbose, "Generating GIF frames...\n")
-        
-        try:
-
-            for frame in self.iter_frames(fps=fps, progress_bar=True):
-                proc1.stdin.write(frame.tostring())
-            verbose_print(verbose, "Done.\n")
-
-        except IOError as err:
-
-            error = ("MoviePy Error: creation of %s failed because "
-              "of the following error:\n\n%s.\n\n."%(filename, str(err)))
-            
-            if program == "ImageMagick":
-                error = error + ("This can be due to the fact that "
-                    "ImageMagick is not installed on your computer, or "
-                    "(for Windows users) that you didn't specify the "
-                    "path to the ImageMagick binary in file conf.py." )
-            
-            raise IOError(error)
-        verbose_print(verbose, "Writing GIF... ")
-        proc1.stdin.close()
-        proc1.wait()
-        if program == 'ImageMagick':
-            proc2.wait()
-            if opt:
-                proc3.wait()
-        verbose_print(verbose, 'Done. Your GIF is ready !')
-
-
+            write_gif(self, filename, fps=fps, program= program,
+              opt=opt, fuzz=fuzz, verbose=verbose, loop=loop,
+              dispose= dispose, colors=colors)
 
     #-----------------------------------------------------------------
     # F I L T E R I N G
@@ -743,8 +559,8 @@ class VideoClip(Clip):
             mask = ColorClip(self.size, 1.0, ismask=True)
             return self.set_mask( mask.set_duration(self.duration))
         else:
-            get_frame = lambda t: np.ones(self.get_frame(t).shape, dtype=float)
-            mask = VideoClip(ismask=True, get_frame = get_frame)
+            make_frame = lambda t: np.ones(self.get_frame(t).shape, dtype=float)
+            mask = VideoClip(ismask=True, make_frame = make_frame)
             return self.set_mask(mask.set_duration(self.duration))
 
 
@@ -803,14 +619,14 @@ class VideoClip(Clip):
 
 
     @outplace
-    def set_get_frame(self, gf):
+    def set_make_frame(self, mf):
         """ Change the clip's ``get_frame``.
 
-        Returns a copy of the VideoClip instance, with the get_frame
-        attribute set to `gf`.
+        Returns a copy of the VideoClip instance, with the make_frame
+        attribute set to `mf`.
         """
-        self.get_frame = gf
-        self.size = gf(0).shape[:2][::-1]
+        self.make_frame = mf
+        self.size = self.get_frame(0).shape[:2][::-1]
 
 
 
@@ -1026,7 +842,7 @@ class ImageClip(VideoClip):
 
         # if the image was just a 2D mask, it should arrive here
         # unchanged
-        self.get_frame = lambda t: img
+        self.make_frame = lambda t: img
         self.size = img.shape[:2][::-1]
         self.img = img
 
@@ -1060,7 +876,7 @@ class ImageClip(VideoClip):
 
         arr = image_func(self.get_frame(0))
         self.size = arr.shape[:2][::-1]
-        self.get_frame = lambda t: arr
+        self.make_frame = lambda t: arr
         self.img = arr
 
         for attr in apply_to:
